@@ -23,7 +23,7 @@ import yaml
 from addict import Dict
 from appdirs import AppDirs
 from requests import Response
-from rich import print
+from rich import print, get_console
 
 from kodexa.connectors import get_source
 from kodexa.connectors.connectors import get_caller_dir, FolderConnector
@@ -140,9 +140,7 @@ class PipelineMetadataBuilder:
 
 DEFAULT_COLUMNS = {
     'extensionPacks': [
-        'orgSlug',
-        'slug',
-        'version',
+        'ref',
         'name',
         'description',
         'type',
@@ -150,8 +148,17 @@ DEFAULT_COLUMNS = {
     ],
     'projects': [
         'id',
+        'organization.name',
         'name',
         'description',
+        'assistants'
+    ],
+    'assistants': [
+        'id',
+        'name',
+        'description',
+        'active',
+        'schedules.nextEvent'
     ],
     'executions': [
         'id',
@@ -162,18 +169,14 @@ DEFAULT_COLUMNS = {
         'documentFamily.path'
     ],
     'stores': [
-        'orgSlug',
-        'slug',
-        'version',
+        'ref',
         'name',
         'description',
         'storeType',
         'storePurpose'
     ],
     'default': [
-        'orgSlug',
-        'slug',
-        'version',
+        'ref',
         'name',
         'description',
         'type'
@@ -230,7 +233,8 @@ OBJECT_TYPES = {
         "name": "execution",
         "plural": "executions",
         "type": Execution,
-        "global": True
+        "global": True,
+        "sort": "startDate:desc"
     }
 }
 
@@ -437,13 +441,18 @@ class KodexaPlatform:
     def __build_object(ref, object_type_metadata, object_class):
         from kodexa import KodexaPlatform
         url = f"{KodexaPlatform.get_url()}/api/{object_type_metadata['plural']}/{ref.replace(':', '/')}"
-
         import requests
         response = requests.get(url,
                                 headers={"x-access-token": KodexaPlatform.get_access_token(),
                                          "content-type": "application/json"})
 
-        return object_class(**response.json())
+        if response.status_code == 200:
+            return object_class(**response.json())
+        elif response.status_code == 404:
+            return None
+        else:
+            raise Exception(
+                f"Unable to create {object_type_metadata['plural']} with ref {ref} [{response.status_code} {response.text}]")
 
     @staticmethod
     def get_object_instance(ref: str, object_type):
@@ -454,13 +463,7 @@ class KodexaPlatform:
 
         if object_type == 'stores':
             # We need to work out what type of store we have
-            obj_info = KodexaPlatform.get_object(ref, object_type)
-            if obj_info['storeType'] == 'TABLE':
-                return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteDataStore)
-            if obj_info['storeType'] == 'DOCUMENT':
-                return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteDocumentStore)
-            if obj_info['storeType'] == 'MODEL':
-                return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteModelStore)
+            return KodexaPlatform.get_object(ref, object_type)
 
         # TODO - there are other things we need?
         raise Exception(f"Unable to get a local instance of {ref} of type {object_type}")
@@ -627,7 +630,7 @@ class KodexaPlatform:
             return list_response.json()
         else:
             logger.warning(list_response.text)
-            raise Exception("Unable to list objects")
+            raise Exception("Unable to list objects [" + list_response.text + "]")
 
     @staticmethod
     def undeploy(object_type: str, ref: str):
@@ -671,9 +674,19 @@ class KodexaPlatform:
             raise Exception(f"Unable to get object {ref}")
         else:
             obj_json = obj_response.json()
+            if obj_json is None:
+                return None
             if 'type' in object_type_metadata:
                 return object_type_metadata['type'].parse_obj(obj_json)
+            if object_type == 'stores':
+                if obj_json['storeType'] == 'TABLE':
+                    return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteDataStore)
+                if obj_json['storeType'] == 'DOCUMENT':
+                    return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteDocumentStore)
+                if obj_json['storeType'] == 'MODEL':
+                    return KodexaPlatform.__build_object(ref, object_type_metadata, RemoteModelStore)
             else:
+                print("errr")
                 return obj_json
 
     @classmethod
@@ -702,6 +715,17 @@ class KodexaPlatform:
         else:
             print(f"Check your URL and password [{obj_response.status_code}]")
 
+    @classmethod
+    def get_project_resource(cls, id, resource_name, resource_type):
+        obj_response = requests.get(f"{KodexaPlatform.get_url()}/api/projects/{id}/{resource_name}",
+                                    headers={"content-type": "application/json",
+                                             "x-access-token": KodexaPlatform.get_access_token()})
+
+        if obj_response.status_code == 200:
+            cls.__print_table(obj_response.json(), resource_type, False, False)
+        else:
+            print(f"Check your URL and password [{obj_response.status_code}]")
+
     @classmethod
     def projects(cls):
         obj_response = requests.get(f"{KodexaPlatform.get_url()}/api/projects",
@@ -739,10 +763,12 @@ class KodexaPlatform:
         if existing is not None:
 
             obj_response = requests.put(f"{KodexaPlatform.get_url()}/api/{object_type}/{url_ref}",
+                                        json=obj,
                                         headers={"x-access-token": KodexaPlatform.get_access_token(),
                                                  "content-type": "application/json"})
         else:
             obj_response = requests.post(f"{KodexaPlatform.get_url()}/api/{object_type}/{url_ref.split('/')[0]}",
+                                         json=obj,
                                          headers={"x-access-token": KodexaPlatform.get_access_token(),
                                                   "content-type": "application/json"})
         if obj_response.status_code != 200:
@@ -761,6 +787,9 @@ class KodexaPlatform:
 
         object_type, object_type_metadata = resolve_object_type(object_type)
 
+        if sort is None and 'sort' in object_type_metadata:
+            sort = object_type_metadata['sort']
+
         if 'global' not in object_type_metadata and not ref:
             print(":fire: You must provide a ref for this type of resource")
             return
@@ -771,6 +800,10 @@ class KodexaPlatform:
             if ref and ('/' in ref or 'global' in object_type_metadata):
                 obj = KodexaPlatform.get_object(ref, object_type)
 
+                if obj is None:
+                    print(f":fire: Unable to find {object_type_metadata['name']} {ref}")
+                    return
+
                 if path is not None:
                     import jq
                     obj = jq.compile(path).input(obj).all()
@@ -780,6 +813,7 @@ class KodexaPlatform:
                         return self.represent_scalar('tag:yaml.org,2002:null', '')
 
                     yaml.add_representer(type(None), represent_none)
+
                     print(yaml.dump(obj.dict(by_alias=True)))
                 elif format == 'json':
                     print(obj.json(by_alias=True, indent=4))
@@ -788,32 +822,7 @@ class KodexaPlatform:
 
             else:
                 objects = KodexaPlatform.list_objects(ref, object_type, query, page, pagesize, sort)
-                cols = DEFAULT_COLUMNS['default']
-
-                if object_type in DEFAULT_COLUMNS:
-                    cols = DEFAULT_COLUMNS[object_type]
-
-                print("\n")
-                from rich.table import Table
-
-                table = Table(title=f"Listing {object_type_metadata['plural']}")
-                for col in cols:
-                    table.add_column(col)
-                for object_dict in objects['content']:
-                    row = []
-
-                    for col in cols:
-                        from simpleeval import simple_eval
-                        from simpleeval import AttributeDoesNotExist
-                        try:
-                            row.append(simple_eval('object.' + col, names={'object': object_dict}))
-                        except AttributeDoesNotExist:
-                            row.append("")
-                    table.add_row(*row)
-
-                print(table)
-
-                print(f"\n{objects['totalElements']} {object_type_metadata['plural']} found, page {objects['number']+1} of {objects['totalPages']}")
+                cls.__print_table(objects, object_type)
         except:
             print(f"\n:exclamation: Failed to get {object_type_metadata['name']} [{sys.exc_info()[0]}]")
             print("\n".join(
@@ -918,17 +927,98 @@ class KodexaPlatform:
     @classmethod
     def logs(cls, execution_id):
         r = requests.get(f"{KodexaPlatform.get_url()}/api/executions/{execution_id}/logs",
+                         params={"page": 1, "pageSize": 10000, "sort": "logDate:asc"},
                          headers={"x-access-token": KodexaPlatform.get_access_token(),
                                   "content-type": "application/json"})
         if r.status_code == 401:
             raise Exception("Your access token was not authorized")
         elif r.status_code == 200:
             for entry in r.json()['content']:
-                print(entry['entry'], end='')
+                get_console().print(entry['entry'], end='', markup=False)
         else:
             logger.warning(r.text)
             raise Exception("Unable to reindex check your reference and platform settings")
 
+    @classmethod
+    def upload_file(cls, ref, path):
+        store = KodexaPlatform.get_object_instance(ref, 'store')
+
+        if isinstance(store, RemoteDocumentStore):
+            with open(path, 'rb') as content:
+                store.put_native(path, content)
+        else:
+            raise Exception("Reference must be a document store")
+
+    @classmethod
+    def get_project(cls, id):
+        project_instance = cls.get_object_instance(id, 'project')
+        print(f"Name: [bold]{project_instance.name}[/bold]")
+        print(f"Description: [bold]{project_instance.description}[/bold]\n")
+
+        print("[bold]Document Stores[/bold]")
+        cls.get_project_resource(id, 'documentStores', 'stores')
+        print("[bold]Data Stores[/bold]")
+        cls.get_project_resource(id, 'dataStores', 'stores')
+        print("[bold]Content Taxonomies[/bold]")
+        cls.get_project_resource(id, 'contentTaxonomies', 'taxonomies')
+        print("[bold]Assistants[/bold]")
+        cls.get_project_resource(id, 'assistants', 'assistants')
+
+    @classmethod
+    def __print_table(cls, objects, object_type, title=True, show_count=True):
+        cols = DEFAULT_COLUMNS['default']
+
+        object_type, object_type_metadata = resolve_object_type(object_type)
+
+        if object_type in DEFAULT_COLUMNS:
+            cols = DEFAULT_COLUMNS[object_type]
+
+        from rich.table import Table
+
+        table = Table(title=f"Listing {object_type_metadata['plural']}" if title else None)
+        for col in cols:
+            table.add_column(col)
+
+        if 'content' in objects:
+            hits = objects['content']
+        else:
+            hits = objects
+
+        for object_dict in hits:
+            row = []
+
+            for col in cols:
+                from simpleeval import simple_eval
+                from simpleeval import AttributeDoesNotExist
+                try:
+                    row.append(str(simple_eval('object.' + col, names={'object': object_dict})))
+                except AttributeDoesNotExist:
+                    row.append("")
+            table.add_row(*row)
+
+        print(table)
+
+        if 'content' in objects and show_count:
+            print(
+                f"\n{objects['totalElements']} {object_type_metadata['plural']} found, page {objects['number'] + 1} of {objects['totalPages']}")
+
+    @classmethod
+    def send_event(cls, project_id, assistant_id, obj):
+        r = requests.post(f"{KodexaPlatform.get_url()}/api/projects/{project_id}/assistants/{assistant_id}/events",
+                          data={
+                              'eventType': obj['eventType'],
+                              'options': json.dumps(obj['options'])
+                          },
+                          headers={"x-access-token": KodexaPlatform.get_access_token()})
+        if r.status_code == 401:
+            raise Exception("Your access token was not authorized")
+        if r.status_code == 200:
+            print(f"Starting an execution with id {r.json()['id']}")
+            return r.json()
+        else:
+            print(r.text)
+            raise Exception("Unable to send event")
+
 
 class RemoteSession:
     """A Session on the Kodexa platform for leveraging pipelines and services"""
@@ -1287,12 +1377,6 @@ class RemoteAction:
 
         return result_document if result_document else document
 
-    def end_processing(self, pipeline_context: PipelineContext):
-        """ """
-
-        # TODO not yet implemented for remote steps
-        pass
-
     def to_configuration(self):
         """Returns a dictionary representing the configuration information for the step
 
